#! /usr/bin/python
# -*- coding: UTF-8 -*-
from __future__ import print_function

import sys, os, os.path, time
from othman.core import othmanCore, searchIndexer

q = othmanCore(False)
ix = searchIndexer(True)
wc = 0
for n,(o,i) in enumerate(q.getAyatIter(1, 6236)):
    for w in i.split():
        ix.addWord(w,n+1)
        wc += 1
d = os.path.dirname(sys.argv[0])
ix.save()
print("got %d words, %d terms (max term length=%d character, term vectors size=%d bytes)." % (wc, ix.terms_count, ix.maxWordLen, ix.term_vectors_size))





#! /usr/bin/python3

import sys
import os
import sqlite3

#os.system("curl -sSL https://github.com/khaledhosny/quran-data/tarball/master | tar -C othman-data -xzf -")
#os.system("mv othman-data/khaledhosny-quran-data-* othman-data/quran-data")

UPDATE_SQL='UPDATE Quran SET othmani=? WHERE id=?'

db=sqlite3.connect("othman-data/quran.db")

#db.execute("BEGIN")
#db.execute("UPDATE Quran SET othmani=''")
#db.execute("COMMIT")
#exit(-1);

db.execute("BEGIN")
counter=1
for i in range(1, 115):
    with open("othman-data/quran-data/quran/{:03d}.txt".format(i), "r") as f:
        for line in f:
            line=line.strip()
            if not line: continue
            print(counter)
            db.execute(UPDATE_SQL, (line, counter))
            counter+=1
db.execute("COMMIT")
db.execute("VACUUM")




#! /usr/bin/python
import sys, os
from distutils.core import setup
from glob import glob

import py2exe

# to create exe type:
# python exe-setup.py py2exe -O2

# NOTE: before you use this tool you should run make to generate index and locale files


locales=map(lambda i: ('locale/' + i, ['' + i + '/othman.mo',]), glob('locale/*/LC_MESSAGES'))
data_files=[('othman-data',
             glob('othman-data/*')),
             ('.',
              ['COPYING'] + glob('LICENSE*') + glob('README*'))]
data_files.extend(locales)

setup (name='Othman', version='0.2.0',
    description='Othman Quran Browser',
    author='Muayyad Saleh Alsadi',
    author_email='alsadi@ojuba.org',
    url='http://othman.ojuba.org/',
    license='Waqf',
    packages=['othman'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        ],

    windows = [
                {
                      'script': 'othman-browser',
                      'icon_resources': [(1, "othman.ico")],
                }
            ],

    options = {
              'py2exe': {
                  'packages':'encodings',
                  'includes': 'cairo, pango, pangocairo, atk, gobject, gio',
              }
          },
    data_files=data_files
    )





# -*- coding: UTF-8 -*-
"""
univaruints is a serialization of integer list
Copyright © 2009-2013, Muayyad Alsadi <alsadi@ojuba.org>



this implementation is unit-tested (by running this module)
"""
import struct, bisect
int64=struct.Struct('>Q')
shifts=[0, 128, 16512, 2113664, 270549120, 34630287488, 4432676798592, 567382630219904, 72624976668147840]
shifts2=shifts[2:]
n_by_chr='\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x01\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x02\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x03\x04\x04\x04\x04\x04\x04\x04\x04\x05\x05\x05\x05\x06\x06\x07\x08'

def bisect_right7(a, x):
    if x<a[3]:
        if x<a[1]:
            if (x<a[0]):
                return 0
            else:
                return 1
        else:
            if x<a[2]:
                return 2
            else:
                return 3
    else:
        if x<a[5]:
            if x<a[4]:
                return 4
            else:
                return 5
        else:
            if x<a[6]:
                return 6
            else:
                return 7



def write(f, s, max_items=0, incremental=0, unique=0, last_item=0):
  """
  encode the sequence s and write the string into the file-like object f

  only max_items are encoded (0 mean infinity)
  if incremental is set to True then the sequence is assumed to be incremental
  which would result in more compact output
  if you know that the sequence is strictly increasing then set unique=1
  """
  # NOTE: below some lookups before loop for optimizations
  fwrite=f.write
  rbisect=bisect_right7 #bisect.bisect_right
  int64pack=int64.pack
  char=chr

  count=0
  last_item-=unique
  for item in s:
    if max_items and count>=max_items: break
    count+=1
    v=item
    if incremental:
      if item<last_item+unique: raise ValueError
      v-=last_item+unique
    if v<128:
      fwrite(char(v))
    else:
      n=rbisect(shifts2, v)+1
      offset=shifts[n]
      v-=offset
      fwrite(char(((0b1111111100000000>>n) & 255) | ( (127>>n) & (v>>(n<<3)) ))+int64pack(v)[8-n:])
    last_item=item
  return count

def read(f, max_items=0, incremental=0, unique=0, last_item=0):
  """
  read and decode sequence at most max_items from the file-like object

  parameters are just like write
  """
  # NOTE: below some lookups before loop for optimizations
  fread=f.read
  int64unpack=int64.unpack
  chr2int=ord

  count=0
  last_item-=unique

  while True:
    if max_items and count>=max_items: break
    ch=fread(1)
    if not ch: break
    count+=1
    o=chr2int(ch)
    if o<128:
      v=o
    else:
      n_ch=n_by_chr[o]
      n=chr2int(n_ch)
      mask=127>>n
      payload=fread(n)
      if len(payload)<n: raise IOError
      v=shifts[n] + (((o & mask)<< (n<<3)) | ( (int64unpack(('\0'*(8-n))+payload))[0] ))
    if incremental: v+=last_item+unique
    yield v
    last_item=v


def decode_single(s):
  """
  return number of bytes consumed and the decoded value
  """   
  o=ord(s[0])
  if o<128: return 1, o
  n_ch=n_by_chr[o]
  n=ord(n_ch)
  mask=127>>n
  return n+1, shifts[n] + (((o & mask)<< (n<<3)) | ( (int64.unpack(('\0'*(8-n))+s[1:n+1]))[0] ))

def decode(s):
  "return a generator that yields all decoded integers"
  offset=0
  while offset<len(s):
    o=ord(chr(s[offset]))
    offset+=1
    if o<128: yield o # just an optimization
    else:
      n_ch=n_by_chr[o]
      n=ord(n_ch)
      mask=127>>n
      yield shifts[n] + (((o & mask)<< (n<<3)) | ( (int64.unpack(('\0'*(8-n)).encode("ISO-8859-1")+s[offset:offset+n]))[0] ))
      offset+=n

def encode_single(v):
    if v<128: return chr(v)
    n=bisect_right7(shifts2, v)+1 #bisect.bisect_right(shifts2, v)+1
    offset=shifts[n]
    v-=offset
    r = chr(((0b1111111100000000>>n) & 255) | ( (127>>n) & (v>>(n<<3)) )).encode("ISO-8859-1") + int64.pack(v)[8-n:]
    return r.decode("ISO-8859-1")

def encode_single_alt(v):
    if v<128: return chr(v) # just an optimization
    offset=128
    m=0
    # enumerate was slower
    #for i,m in enumerate(shifts2):
    for i in shifts2: # although we can use bisect, but we only got 8 elements
        n=m+1
        if v<i:
            v-=offset
            msb=((0b1111111100000000>>n) & 255) | ( (127>>n) & (v>>(n<<3)) )
            p=int64.pack(v)
            return chr(msb) + p[8-n:]
        offset=i
        m+=1
    #m+=1 # if enumerate is used uncomment this line
    v-=offset
    n=m+1
    msb=((0b1111111100000000>>n) & 255) | ( (127>>n) & (v>>(n<<3)) )
    p=int64.pack(v)
    return chr(msb) + p[8-n:]

def encode(a):
    return "".join(map(encode_single, a))

def incremental_encode_list(a, unique=1, last=0):
  if unique!=1 and unique!=0: raise ValueError
  last-=unique
  for i in a:
    if i<last+unique: raise ValueError
    yield i-last-unique
    last=i

def incremental_decode_list(a, unique=1, last=0):
  if unique!=1 and unique!=0: raise ValueError
  last-=unique
  for i in a:
    j=i+last+unique
    yield j
    last=j

def incremental_encode(a, unique=1, last=0):
  return encode(incremental_encode_list(a, unique, last))

def incremental_decode(s, unique=1, last=0):
  return incremental_decode_list(decode(s), unique, last)

#import unittest
#class TestSequenceFunctions(unittest.TestCase):
#    def setUp(self):
#        self.seq = range(10)
#    def test_t1(self):
#        self.assertEqual(x, y)
#        self.assertRaises(TypeError, random.shuffle, (1,2,3))
#        self.assertTrue(element in self.seq)
# in main run unittest.main()

if __name__ == "__main__":
  import time, itertools, random
  from cStringIO import StringIO
  boundary=[(i-1,i,i+1) for i in shifts[1:]]
  boundary=list(itertools.chain(*boundary))
  boundary.insert(0,0)
  print ("simple unit tests...")
  l1=[0,1,100,200,300,500,1000,10000]
  for i in l1:
    print ('before dec:', i, ', hex:', hex(i), ', bin:', bin(i))
    e=encode_single(i)
    print ('after len:',len(e), ', str:', repr(e))
    assert i == decode_single(encode_single(i))[1]
  f=StringIO()
  write(f, l1)
  f.seek(0)
  assert l1==list(read(f))
  print ("boundary unit tests...")
  for i in boundary:
    print ('before dec:', i, ', hex:', hex(i), ', bin:', bin(i))
    e=encode_single(i)
    print ('after len:',len(e), ', str:', repr(e))
    assert i == decode_single(encode_single(i))[1]
  assert boundary == list(decode(encode(boundary)))
  assert boundary == list(incremental_decode(incremental_encode(boundary, unique=0), unique=0))
  assert boundary == list(incremental_decode(incremental_encode(boundary, unique=1), unique=1))
  
  f=StringIO()
  write(f, boundary, 0, 0, 0)
  f.seek(0)
  assert boundary == list(read(f, 0, 0, 0))
  f=StringIO()
  write(f, boundary, 0, 1, 0)
  f.seek(0)
  assert boundary == list(read(f, 0, 1, 0))
  f=StringIO()
  write(f, boundary, 0, 1, 1)
  f.seek(0)
  assert boundary == list(read(f, 0, 1, 1))


  print ("random unit tests...")
  l=[random.randint(0, 5000000) for i in range(1000)]
  s=encode(l)
  l2=list(decode(s))
  assert l2==l
  ll=0
  l=[0]
  for i in range(1000):
    ll+=random.randint(0, 5000000)
    l.append(ll)
  l2=list(incremental_decode(incremental_encode(l, unique=0), unique=0))
  assert l2==l

  ll=0
  l=[0]
  for i in range(1000):
    ll+=random.randint(1, 5000000)
    l.append(ll)
  l2=list(incremental_decode(incremental_encode(l, unique=1), unique=1))
  assert l2==l

  print ("pass")
  print ("performance tests")
  q=struct.Struct('>Q')
  pack=lambda l: ''.join(itertools.imap(lambda i: q.pack(i), l))
  def unpack(s):
      for i in range(0,len(s),8):
          yield q.unpack(s[i:i+8])[0]
  t1=time.time()
  for i in range(1000): list(unpack(pack(boundary)))
  t2=time.time()
  delta_pack=t2-t1
  print ('struct-based done in ', delta_pack)

  f=StringIO()
  t1=time.time()
  for i in range(1000):
    f.seek(0)
    write(f, boundary)
    f.seek(0)
    list(read(f))
  t2=time.time()
  print ('file-like done in ', t2-t1)


  t1=time.time()
  for i in range(1000): list(decode(encode(boundary)))
  t2=time.time()
  delta_our=t2-t1
  print ('we are done in ', delta_our)
  t1=time.time()
  for i in range(1000): encode(boundary)
  t2=time.time()
  delta_our=t2-t1
  print ('we are done in encoding in ', delta_our)
  e=encode(boundary)
  t1=time.time()
  for i in range(1000): list(decode(e))
  t2=time.time()
  delta_our=t2-t1
  print ('we are done in decoding in ', delta_our)




# -*- coding: UTF-8 -*-
"""
Othman - Quran browser
Core - python module for accessing Othman API

Copyright © 2009-2010, Muayyad Alsadi <alsadi@ojuba.org>

    Released under terms of Waqf Public License.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the latest version Waqf Public License as
    published by Ojuba.org.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

    The Latest version of the license can be found on
    "http://waqf.ojuba.org/license"

"""
import sys, os, os.path, time
import sqlite3
import array
#from itertools import imap
import threading
from functools import reduce
from .univaruints import *
try:
    from itertools import imap
except:
    imap = lambda *args, **kwargs: list(map(*args, **kwargs))



data_dir = None

def cmp(a, b):
    return (a > b) - (a < b)

def guessDataDir():
    global data_dir
    if data_dir: return data_dir
    if not hasattr(sys, "frozen"):
        # we are not in py2exe
        d = os.path.join('share', 'othman')
        if os.path.exists(d):
            data_dir=os.path.abspath(os.path.realpath(d))
            return data_dir
        f = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))
        d = os.path.join(f, '..', 'othman-data')
        if os.path.exists(d):
            data_dir=os.path.abspath(os.path.realpath(d))
            return data_dir
        d = os.path.join(f,'..', '..', '..', '..', 'share', 'othman')
        if os.path.exists(d):
            data_dir = os.path.abspath(os.path.realpath(d))
            return data_dir
    # we are in py2exe or DATA can't be located relative to __FILE__
    f = os.path.abspath(os.path.realpath(os.path.dirname(sys.argv[0])))
    d = os.path.join(f, 'othman-data')
    if os.path.exists(d):
        data_dir=os.path.abspath(os.path.realpath(d))
        return data_dir
    d = os.path.join(f, '..', 'share', 'othman')
    data_dir = os.path.abspath(os.path.realpath(d))
    return data_dir

def cmp_bisect_right(ccmp, a, x, lo=0, hi=None):
    """
        same as bisect.bisect but uses custom cmp function
    """
    if lo < 0:
        raise ValueError('lo must be non-negative')
    if hi is None:
        hi = len(a)
    while lo < hi:
        mid = (lo + hi) >> 1
        if ccmp(a[mid], x) > 0:
            hi = mid # ie. if x < a[mid]
        else:
            lo = mid + 1
    return lo

def cmp_bisect_left(ccmp, a, x, lo=0, hi=None):
    """
        same as bisect.bisect_left but uses custom cmp function
    """
    if lo < 0:
        raise ValueError('lo must be non-negative')
    if hi is None:
        hi = len(a)
    while lo < hi:
        mid = (lo + hi) >> 1
        if ccmp(x, a[mid]) > 0:
            lo = mid + 1 # ie. if a[mid] < x
        else:
            hi = mid
    return lo


class othmanCore(object):
    SQL_GET_AYAT = 'SELECT othmani, imlai FROM Quran WHERE id>=? ORDER BY id LIMIT ?'
    SQL_GET_SURA_INFO = 'SELECT rowid, sura_name, other_names, makki, starting_row, comment FROM SuraInfo ORDER BY rowid'
    def __init__(self, load_ix=True):
        self.data_dir = d = guessDataDir()
        self.db_fn = db_fn = os.path.join(d, 'quran.db')
        self._cn = {}
        cn = self._getConnection()
        l = list(cn.execute(self.SQL_GET_SURA_INFO).fetchall())
        if len(l) != 114:
            raise IOError
        self.suraIdByName = dict(((i[1], i[0]) for i in l))
        self.suraInfoById = [list(i[1:]) + [0] for i in l]
        for i in range(113):
            self.suraInfoById[i][5] = self.suraInfoById[i + 1][3] - self.suraInfoById[i][3]
        self.suraInfoById[-1][5] = 6
        self.basmala, self.basmala_imlai = list(self.getAyatIter(1))[0]
        self.basmala = self.basmala[:-3]
        self.ix = None
        if load_ix:
            self.ix = searchIndexer()

    def _getConnection(self):
        n = threading.current_thread().name
        if n in self._cn.keys():
            r = self._cn[n]
        else:
            r = sqlite3.connect(self.db_fn)
            self._cn[n] = r
        return r

    def showSunnahBasmala(self, sura):
        return sura != 1 and sura != 9

    def suraAyaFromAyaId(self, ayaId):
        sura = cmp_bisect_right(lambda i, j: cmp(i[3], j), self.suraInfoById, ayaId)
        aya = ayaId - self.suraInfoById[sura-1][3] + 1
        return sura, aya

    def ayaIdFromSuraAya(self, suraId, aya = 1):
        """
            suraId: sura number from 1 to 114
            aya: aya number from 1 to the end of the sura
        """
        return self.suraInfoById[suraId-1][3] + aya - 1

    def getAyatIter(self, ayaId, number = 1):
        """
            return a list of (othmani, imlai) tuples
        """
        return self._getConnection().execute(self.SQL_GET_AYAT, (ayaId,number))

    def getSuraIter(self, suraId, number=0, fromAya=1):
        """
            return a list of (othmani, imlai) tuples for the whole sura
            suraId: sura number from 1 to 114
            number of ayat (0 for the whole sura)
        """
        a = self.suraInfoById[suraId - 1]
        m = a[5]
        if number <= 0 or number > m:
            number = m
        return self.getAyatIter(a[3] + fromAya - 1, number)

normalize_tb={
    65: 97, 66: 98, 67: 99, 68: 100, 69: 101, 70: 102,
    71: 103, 72: 104, 73: 105, 74: 106, 75: 107, 76: 108,
    77: 109, 78: 110, 79: 111, 80: 112, 81: 113, 82: 114,
    83: 115, 84: 116, 85: 117, 86: 118, 87: 119, 88: 120,
    89: 121, 90: 122, 1600: None, 1569: 1575, 1570: 1575,
    1571: 1575, 1572: 1575, 1573: 1575, 1574: 1575, 1577: 1607,
    1611: None, 1612: None, 1613: None, 1614: None, 1615: None,
    1616: None, 1617: None, 1618: None, 1609: 1575}

def normalize(s):
    return s.translate(normalize_tb)

class searchIndexerItem(set):
    def __init__(self, *a):
        set.__init__(self, *a)

    def __or__(self, y):
        return self.union(y)

    def __and__(self, y):
        return self.intersection(y)

    def toAyaIdList(self):
        l = list(self)
        l.sort()
        return l

class searchIndexer:
    def __init__(self, unlink = False, normalize = normalize):
        d = guessDataDir()
        self.db_fn = fn = os.path.join(d, "ix.db")
        if unlink and os.path.exists(fn):
            os.unlink(fn)
        self._cn = {}
        self.d = {}
        self.normalize = normalize
        self.maxWordLen = 0
        self.term_vectors_size = 0
        self.terms_count = 0

    def _getConnection(self):
        n = threading.current_thread().name
        if n in self._cn.keys():
            r = self._cn[n]
        else:
            r = sqlite3.connect(self.db_fn)
            self._cn[n] = r
        return r

    def save(self):
        cn = self._getConnection()
        c = cn.cursor()
        c.execute('CREATE TABLE ix (w TEXT PRIMARY KEY NOT NULL, i BLOB)')
        for w in self.d:
            b = incremental_encode(self.d[w].toAyaIdList())
            self.term_vectors_size += len(b)
            c.execute( 'INSERT INTO ix VALUES(?,?)', (w, sqlite3.Binary(b.encode("ISO-8859-1"))) )
        self.terms_count = len(self.d.keys())
        cn.commit()

    def _itemFactory(self, r):
        a = incremental_decode(r[1])
        return r[0], searchIndexerItem(a)

    def _itemFactory2(self, r):
        a = incremental_decode(r[1])
        return searchIndexerItem(a)

    def get(self, w):
        cn = self._getConnection()
        r = cn.execute('SELECT w, i FROM ix WHERE w=?', (w,)).fetchone()
        if not r:
            return None, None
        return self._itemFactory(r)

    def getPartial(self, w, withWords=False):
        if "%" in w or "_" in w:
            return [] # special chars
        cn = self._getConnection()
        W = "%" + w + "%"
        f = withWords and self._itemFactory or self._itemFactory2
        r = cn.execute('SELECT w, i FROM ix WHERE w LIKE ?', (W, ))
        if not r:
            return []
        return imap(lambda i: f(i), r)

    def find(self, words):
        if not words:
            return None
        w = self.normalize(words[0])
        W, x = self.get(w)
        if not x:
            return None
        for w in words[1:]:
            W, y = self.get(self.normalize(w))
            if not y:
                return None
            x &= y
        return x.toAyaIdList()

    def findPartial(self, words):
        if not words:
            return None
        w = self.normalize(words[0])
        x = reduce( lambda a,b: a|b, self.getPartial(w), searchIndexerItem() )
        for W in words[1:]:
            w = self.normalize(W)
            y = reduce( lambda a,b: a|b, self.getPartial(w), searchIndexerItem() )
            x &= y
        try :
            return x.toAyaIdList()
        except AttributeError:
            x2 = searchIndexerItem(x)
            return x2.toAyaIdList()

    def addWord(self, word, ayaId):
        w = self.normalize(word)
        #if not w: print word; return
        self.maxWordLen = max(self.maxWordLen,len(w))
        if w in self.d.keys():
            self.d[w].add(ayaId)
        else:
            self.d[w] = searchIndexerItem((ayaId,))





# -*- coding: UTF-8 -*-
"""
Othman - Quran browser
gtkUi - gtk user interface for Othman API

Copyright © 2008-2010, Muayyad Alsadi <alsadi@ojuba.org>

        Released under terms of Waqf Public License.
        This program is free software; you can redistribute it and/or modify
        it under the terms of the latest version Waqf Public License as
        published by Ojuba.org.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

        The Latest version of the license can be found on
        "http://waqf.ojuba.org/license"

"""
import gi
gi.require_version('Gtk', '3.0')
import sys, os, os.path, time
import gettext

import gi
gi.require_version("Gtk", "3.0")

from gi.repository import Gtk, Gdk, GLib, Pango, GdkPixbuf
from .core import othmanCore, searchIndexer

PY2 = sys.version_info[0]==2
BYTE = str if PY2 else bytes

class searchWindow(Gtk.Window):
    def __init__(self, w):
        Gtk.Window.__init__(self)
        self.w = w
        self.connect('delete-event', lambda w,*a: w.hide() or True)
        self.last_txt = None
        self.set_type_hint(Gdk.WindowTypeHint.UTILITY)
        self.set_modal(True)
        self.set_deletable(True)
        self.set_title(_('Search results'))
        self.set_transient_for(w)
        vb = Gtk.VBox(False,0)
        self.add(vb)
        
        self.search = Gtk.Entry()
        self.search.set_width_chars(15)
        vb.pack_start(self.search, False,False, 0)
        
        self.scroll = Gtk.ScrolledWindow()
        self.scroll.set_policy(Gtk.PolicyType.AUTOMATIC,Gtk.PolicyType.AUTOMATIC)
        self.scroll.set_size_request(100, 250)
        vb.pack_start(self.scroll,True, True, 6)
        
        self.ls = Gtk.ListStore(int,str,int,int)
        self.cells = []
        self.cols = []
        self.cells.append(Gtk.CellRendererText())
        self.cols.append(Gtk.TreeViewColumn(_('Sura'), self.cells[0], text=1))
        self.cols[0].set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
        self.cols[0].set_resizable(True)

        self.cells.append(Gtk.CellRendererText()); # self.cols[-1].set_expand(False)
        self.ls_w = Gtk.TreeView(self.ls)
        self.ls_w.connect("cursor-changed", self.move)
        self.ls_w.set_direction(Gtk.TextDirection.RTL)
        self.ls_w.set_headers_visible(False)
        for i in self.cols:
            self.ls_w.insert_column(i, -1)
        self.scroll.add(self.ls_w)
        self.search.connect("activate", self.search_cb)
        self.show_all()

    def move(self, t):
        a = self.ls_w.get_selection().get_selected()
        if not a or len(a) < 2 or not a[1]:
            return
        sa = self.ls[self.ls.get_path(a[1])]
        self.w.sura_c.set_active(sa[2]-1)
        self.w.viewAya(sa[3], sa[2])

    def search_cb(self, b, *a):
        t = b.get_text()
        self.w.search.set_text(t)
        self.find(t)

    def find(self, txt, backward = False):
        txt = txt.strip()
        if not txt:
            self.hide()
            return
        if type(txt) == BYTE:
            txt = txt.decode('utf-8')
        if txt == self.last_txt:
            # TODO: just move cursor to next/prev result before showing it
            pass
        else:
            self.search.set_text(txt)
            self.last_txt = txt
            self.ls.clear()
            for i in self.w.ix.findPartial(txt.split()):
                sura, aya = self.w.suraAyaFromAyaId(i)
                name = self.w.suraInfoById[sura-1][0]
                self.ls.append([i, "%03d %s - %03d" % (sura, name, aya), sura, aya,])
            self.ls_w.set_cursor(Gtk.TreePath(path=0), None, False)
        self.show_all()

class othmanUi(Gtk.Window, othmanCore):
    def __init__(self):
        Gtk.Window.set_default_icon_name('Othman')
        Gtk.Window.__init__(self)
        othmanCore.__init__(self)
        self.sw = None
        self.lastSearchText = None
        self.lastSearchResult = []
        self.ix = searchIndexer()
        self.set_title(_('Othman Quran Browser'))
        self.connect("delete_event", self.quit)
        self.connect('destroy', self.quit)
        self.set_default_size(600, 480)
        self.maximize()
        self.clip1 = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
        self.clip2 = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        self.accel = Gtk.AccelGroup()

        vb = Gtk.VBox(False,0)
        self.add(vb)
        hb = Gtk.HBox(False,2)
        vb.pack_start(hb, False, False, 0)

        self.scroll = Gtk.ScrolledWindow()
        self.scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.ALWAYS)
        self.scroll.connect_after("size-allocate", self.resize_cb)
        vb.pack_start(self.scroll, True, True, 6)

        img = Gtk.Image()
        img.set_from_stock(Gtk.STOCK_COPY, Gtk.IconSize.BUTTON)
        b = Gtk.Button()
        b.add(img)
        b.connect("clicked", self.show_cp_dlg)
        hb.pack_start(b,False, False, 0)
        hb.pack_start(Gtk.VSeparator(), False, False, 6)
        hb.pack_start(Gtk.Label(_("Sura")), False, False, 0)

        self.sura_ls = tuple("%d. %s" % (i+1,j[0]) for (i,j) in enumerate(self.suraInfoById))
        self.sura_c = Gtk.ComboBoxText.new()
        self.sura_c.set_wrap_width(5)
        for i in self.sura_ls:
            self.sura_c.append_text(i)
        self.sura_c.set_tooltip_text(_("choose a Sura"))
        self.sura_c.connect("changed", self.sura_changed_cb)
        hb.pack_start(self.sura_c, False, False, 0)

        hb.pack_start(Gtk.VSeparator(),False, False, 6)
        img = Gtk.Image()
        img.set_from_stock(Gtk.STOCK_ZOOM_IN, Gtk.IconSize.BUTTON)
        b = Gtk.Button()
        b.add(img)
        hb.pack_start(b, False, False, 0)
        b.connect("clicked", self.zoomIn)
        
        img = Gtk.Image()
        img.set_from_stock(Gtk.STOCK_ZOOM_OUT, Gtk.IconSize.BUTTON)
        b = Gtk.Button()
        b.add(img)
        hb.pack_start(b, False, False, 0)
        b.connect("clicked", self.zoomOut)

        img = Gtk.Image()
        img.set_from_stock(Gtk.STOCK_MEDIA_FORWARD, Gtk.IconSize.BUTTON)
        b = Gtk.ToggleButton()
        b.add(img)
        hb.pack_start(b, False, False, 0)
        self.autoScrolling = False
        b.connect("clicked", self.autoScrollCb)
        GLib.timeout_add(100, self.autoScroll, b)

        hb.pack_start(Gtk.VSeparator(), False, False, 6)
        hb.pack_start(Gtk.Image.new_from_stock(Gtk.STOCK_FIND, Gtk.IconSize.BUTTON), False, False, 0)
        search = Gtk.Entry(); search.set_width_chars(15)
        hb.pack_start(search, False,False, 0)
        search.connect("activate", self.search_cb)
        self.search = search

        hb.pack_start(Gtk.VSeparator(),False, False, 6)
        img = Gtk.Image()
        img.set_from_stock(Gtk.STOCK_ABOUT, Gtk.IconSize.BUTTON)
        b = Gtk.Button()
        b.add(img)
        hb.pack_start(b, False, False, 0)
        b.connect("clicked", lambda *a: self.show_about_dlg(self))


        self.scale = 1
        self.txt = Gtk.ListStore(str,int,str)
        self.cells = []
        self.cols = []
        self.cells.append(Gtk.CellRendererText())
        #self.cols.append(Gtk.TreeViewColumn('Quranic Text', self.cells[0], markup=0))
        self.cols.append(Gtk.TreeViewColumn('Quranic Text', self.cells[0], text = 0, foreground = 2))
        self.cells[0].set_property("background","#fffff8")
        #self.cells[0].set_property("foreground","#204000")
        #self.cells[0].set_property("alignment",Pango.ALIGN_CENTER)
        self.cells[0].set_property("wrap-mode", Pango.WrapMode.WORD)
        # TODO: on-max-min also readjust this value
        self.cells[0].set_property("wrap-width", 300)
        self.cells[0].set_property("font", "Amiri Quran 32")
        #self.cells[0].set_property("font", "Simplified Naskh 32")
        #self.cells[0].set_property("font","KFGQPC Uthmanic Script HAFS 32")
        self.cells[0].set_property("scale", self.scale)
        self.cols[0].set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)

        self.cells.append(Gtk.CellRendererText()); # self.cols[-1].set_expand(False)
        self.txt_list = Gtk.TreeView(self.txt)
        self.txt_list.set_headers_visible(False)
        self.txt_list.set_direction(Gtk.TextDirection.RTL)
        for i in self.cols:
            self.txt_list.insert_column(i, -1)

        self.scroll.add(self.txt_list)
        self.sura_c.set_active(0)
        self.build_cp_dlg()
        self.show_all()

    def show_about_dlg(self, parent):
        dlg = Gtk.AboutDialog()
        dlg.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        dlg.set_modal(True)
        dlg.set_transient_for(parent)
        dlg.set_default_response(Gtk.ResponseType.CLOSE)
        dlg.connect('delete-event', lambda w,*a: w.hide() or True)
        dlg.connect('response', lambda w,*a: w.hide() or True)
        try:
            dlg.set_program_name("Othman")
        except:
            pass
        dlg.set_name(_('Othman Quran Browser'))
        #dlg.set_version(version)
        dlg.set_copyright("Copyright © 2008-2010 Muayyad Saleh Alsadi <alsadi@ojuba.org>")
        dlg.set_comments(_("Electronic Mus-haf"))
        dlg.set_license("""
        Released under terms of Waqf Public License.
        This program is free software; you can redistribute it and/or modify
        it under the terms of the latest version Waqf Public License as
        published by Ojuba.org.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

        The Latest version of the license can be found on
        "http://waqf.ojuba.org/"

        """)
        dlg.set_website("http://othman.ojuba.org/")
        dlg.set_website_label("http://othman.ojuba.org")
        dlg.set_authors(["Muayyad Saleh Alsadi <alsadi@ojuba.org>"])
        dlg.set_translator_credits(_("translator-credits"))
        fn = os.path.join(self.data_dir, "quran-kareem.svg")
        try:
            logo = GdkPixbuf.Pixbuf.new_from_file_at_size(fn, 128, 128)
        except:
            fn = os.path.join(self.data_dir, "quran-kareem.png")
            logo = GdkPixbuf.pixbuf_new_from_file(fn)
        dlg.set_logo(logo)
        #dlg.set_logo_icon_name('Othman')
        dlg.run()
        dlg.destroy()

    def search_cb(self, b, *a):
        if not self.sw:
            self.sw = searchWindow(self)
        self.sw.find(b.get_text())

    def autoScroll(self, b):
        if not self.autoScrolling:
            return True
        v = self.scroll.get_vadjustment()
        m = v.get_upper() - v.get_page_size()
        n = min(m, v.get_value() + 2 )
        if n == m:
            b.set_active(False)
        v.set_value(n)
        return True

    def autoScrollCb(self, b, *a):
        self.autoScrolling = b.get_active()

    def zoomIn(self, *a):
        sura, aya = self.getCurrentSuraAya()
        self.scale += 0.1
        self.cells[0].set_property("scale", self.scale)
        self.resize_cb()
        self.queue_draw()
        self.viewSura(sura)
        self.viewAya(aya)

    def zoomOut(self, *a):
        sura, aya = self.getCurrentSuraAya()
        self.scale -= 0.1
        self.scale = max(0.2, self.scale)
        self.cells[0].set_property("scale", self.scale)
        self.resize_cb()
        self.queue_draw()
        self.viewSura(sura)
        self.viewAya(aya)

    def viewAya(self, aya, sura = None):
        if sura == None:
            sura = self.sura_c.get_active() + 1
        aya = max(1,abs(aya))
        i = aya + int(self.showSunnahBasmala(sura))
        self.txt_list.scroll_to_cell((i - 1,))
        self.txt_list.get_selection().select_path((i - 1,))

    def viewSura(self, i):
        #self.play_pause.set_active(False)
        self.txt.clear()
        if self.showSunnahBasmala(i):
            #self.txt.append(['<span foreground="#440000">%s</span>' % self.basmala,0,])
            self.txt.append([self.basmala, 0, "#802000",])
        for j, k in enumerate(self.getSuraIter(i)):
            self.txt.append([k[0], j + 1, "#204000",])
        self.resize_cb()
        self.scroll.get_vadjustment().set_value(0)
        self.txt_list.get_selection().select_path((0,))

    def sura_changed_cb(self, c, *a):
        self.viewSura(self.sura_c.get_active() + 1)

    def resize_cb(self,*args):
        if self.cols[0].get_width() > 10:
            self.cells[0].set_property("wrap-width", self.cols[0].get_width() - 10)

    def cp_cb(self, *a):
        sura = self.cp_sura.get_active() + 1
        aya1 = self.cp_from.get_value()
        aya2 = self.cp_to.get_value()
        n = aya2 - aya1 + 1
        i = self.cp_is_imlai.get_active()
        a = [' ', '\n', ' * ', ' *\n']
        s = a[int(i) * 2 + int(self.cp_aya_perline.get_active())]
        s = s.join([l[i] for l in self.getSuraIter(sura, n, aya1)]) + '\n'
        self.clip1.set_text(s, -1)
        self.clip2.set_text(s, -1)
        self.cp_w.hide()

    def cp_sura_cb(self, *a):
        sura = self.cp_sura.get_active() + 1
        m = self.suraInfoById[sura - 1][5]
        self.cp_from.set_range(1, m)
        self.cp_to.set_range(1, m)
        self.cp_from.set_value(1)
        self.cp_to.set_value(m)

    def show_cp_dlg(self, *a):
        sura, aya = self.getCurrentSuraAya()
        aya = max(1, abs(aya))
        self.cp_sura.set_active(sura - 1)
        self.cp_sura_cb()
        self.cp_from.set_value(aya)
        self.cp_w.show_all()

    def build_cp_dlg(self):
        self.cp_w = Gtk.Window()
        self.cp_w.set_title(_('Copy to clipboard'))
        self.cp_w.set_type_hint(Gdk.WindowTypeHint.DIALOG)
        self.cp_w.connect('delete-event', lambda w,*a: w.hide() or True)
        self.cp_sura = Gtk.ComboBoxText.new()
        self.cp_sura.set_wrap_width(5)

        for i in self.sura_ls:
            self.cp_sura.append_text(i)
        self.cp_sura.set_tooltip_text(_("choose a Sura"))
        adj = Gtk.Adjustment(0, 0, 286, 1, 10, 0)
        self.cp_from = s = Gtk.SpinButton()
        s.set_adjustment(adj)
        adj = Gtk.Adjustment(0, 0, 286, 1, 10, 0)
        self.cp_to = s = Gtk.SpinButton()
        s.set_adjustment(adj)
        self.cp_is_imlai = Gtk.CheckButton(_("Imla'i style"))
        self.cp_aya_perline = Gtk.CheckButton(_("an Aya per line"))
        self.cp_ok = Gtk.Button(stock=Gtk.STOCK_OK)
        self.cp_cancel = Gtk.Button(stock=Gtk.STOCK_CANCEL)
        vb = Gtk.VBox(False,0)
        self.cp_w.add(vb)
        hb = Gtk.HBox(False,3)
        vb.pack_start(hb,True,True,3)
        hb.pack_start(Gtk.Label(_("Sorat:")),False,False,3)
        hb.pack_start(self.cp_sura,False,False,6)
        hb = Gtk.HBox(False,6)
        vb.pack_start(hb,True,True,6)
        hb.pack_start(Gtk.Label(_("Ayat from:")),False,False,3)
        hb.pack_start(self.cp_from,False,False,3)
        hb.pack_start(Gtk.Label(_("To")),False,False,3)
        hb.pack_start(self.cp_to,False,False,3)
        hb = Gtk.HBox(False,6)
        vb.pack_start(hb,True,True,6)
        hb.pack_start(self.cp_is_imlai,False,False,3)
        hb.pack_start(self.cp_aya_perline,False,False,3)
        hb = Gtk.HBox(False,6)
        vb.pack_start(hb,True,True,6)
        hb.pack_start(self.cp_ok,False,False,6)
        hb.pack_start(self.cp_cancel,False,False,6)
        self.cp_sura.connect("changed", self.cp_sura_cb)
        self.cp_cancel.connect('clicked', lambda *args: self.cp_w.hide())
        self.cp_ok.connect('clicked', self.cp_cb)
        self.cp_is_imlai.set_active(True)

    def getCurrentSuraAya(self):
        a = self.txt_list.get_selection().get_selected()
        aya = 1
        if a:
            aya = self.txt[self.txt.get_path(a[1])][1]
        aya = max(aya, 1)
        return self.sura_c.get_active() + 1, aya

    def quit(self,*args):
         Gtk.main_quit()
         return False

def main():
    exedir = os.path.dirname(sys.argv[0])
    ld = os.path.join(exedir,'..', 'share', 'locale')
    if not os.path.exists(ld):
        ld = os.path.join(exedir, 'locale')
    gettext.install('othman', ld)
    w = othmanUi()
    Gtk.main()

if __name__ == "__main__":
    main()




#! /usr/bin/python3
import sys, os
from distutils.core import setup
from glob import glob

from othman import __version__

# to install type: 
# python setup.py install --root=/

from distutils.command.build import build
from distutils.command.clean import clean

class my_build(build):
  def run(self):
    build.run(self)
    # generate data
    from othman.core import othmanCore, searchIndexer

    if not os.path.isfile('othman-data/ix.db'):
      q=othmanCore(False)
      ix=searchIndexer(True)
      for n,(o,i) in enumerate(q.getAyatIter(1, 6236)):
        for w in i.split(): ix.addWord(w,n+1)
      d=os.path.dirname(sys.argv[0])
      ix.save()

class my_clean(clean):
  def run(self):
    clean.run(self)
    try: os.unlink('othman-data/ix.db')
    except OSError: pass

locales=map(lambda i: ('share/'+i,[''+i+'/othman.mo',]),glob('locale/*/LC_MESSAGES'))
data_files=[('share/othman/',glob('othman-data/*'))]
data_files.extend(locales)
setup (name='Othman', version=__version__,
      description='Othman Quran Browser',
      author='Muayyad Saleh Alsadi',
      author_email='alsadi@ojuba.org',
      url='http://othman.ojuba.org/',
      license='Waqf',
      packages=['othman'],
      scripts=['othman-browser'],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: End Users/Desktop',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          ],
      cmdclass={'build': my_build, 'clean': my_clean},
      data_files=data_files
)

